from pdb import set_trace

import numpy as np
import gymnasium as gym
import pygame
import matplotlib.pyplot as plt

# NOTE: The default behavior is for the pygame screen to display a red screen (never updated
#       with the image or green background) while matplotlib displays correct mujoco images.

info_str = ""
# Steps to take in environment
steps = 100
info_str += f"steps={steps}"
# Determines if first/init env.render() method is called (i.e., outside the step loop)
# NOTE: If enabled, pygame screen will remain red but project an empty image over the screen.
#       Additionally, matplotlib will display a black image as well as image values are all 0.
env_init_render = False
info_str += f"\nenv_init_render={env_init_render}"
# Determines if ALL env.render() method is called
# NOTE: If disabled, pygame screen will display red and then green correctly but no 
#       image will be disabled as env.render() is not called
env_render = True
info_str += f"\nenv_render={env_render}"
# Enable image rendering via pygame
use_pygame = True
info_str += f"\nuse_pygame={use_pygame}"
# Enable image rendering via matplotlib
use_mat = True
info_str += f"\nuse_mat={use_mat}"

print(info_str)

def pygame_init():
    pygame.init()
    pygame.display.init()
    display = (800, 600)
    screen = pygame.display.set_mode(display)
    return screen

env = gym.make("Ant-v4", render_mode="rgb_array")
env.reset()

if env_init_render and env_render:
    print("Running initial env.render()")
    env.render()
    
if use_pygame:
    screen = pygame_init()

if use_mat:
    fig = plt.figure()
    viewer = fig.add_subplot(111)

for i in range(steps):
    state_tuple = env.step(env.action_space.sample())
    
    # Render pygame screen green on first step to check when Pygame display is working
    # If it renders black or stays red, then screen is not being updated correctly
    # Thus, the first step should render a red screen followed by a green screen or the actual images
    if use_pygame:
        if i==0:
            screen.fill((255, 0, 0))
            pygame.display.flip() 
            pygame.time.wait(1000)
        else:       
            screen.fill((0, 255, 0))
            
    if env_render:      
        img = env.render()
        
        if i == steps//2:
            print(f"Max pixel: {np.max(img)}")
        
        if use_pygame:
            pyg_img = pygame.surfarray.make_surface(img)
            screen.blit(pyg_img, (0, 0))
        if use_mat:
            viewer.clear() 
            viewer.imshow(img)
            plt.pause(.01)
            fig.canvas.draw()
    
    if use_pygame:
        pygame.display.flip()